@@ -16,7 +16,7 @@
 #include <QRegularExpression>
 
 KConfigSourceGenerator::KConfigSourceGenerator(const QString &inputFile, const QString &baseDir, const KConfigParameters &cfg, ParseResult &parseResult)
-    : KConfigCodeGeneratorBase(inputFile, baseDir, baseDir + cfg.baseName + QLatin1Char('.') + cfg.sourceExtension, cfg, parseResult)
+    : KConfigCodeGeneratorBase(inputFile, baseDir, baseDir + cfg.baseName + QLatin1String(".cpp"), cfg, parseResult)
 {
 }
 
@@ -40,7 +40,6 @@ void KConfigSourceGenerator::start()
     createDestructor();
     createNonModifyingSignalsHelper();
     createSignalFlagsHandler();
-    includeMoc();
 }
 
 void KConfigSourceGenerator::createHeaders()
@@ -77,6 +76,10 @@ void KConfigSourceGenerator::createHeaders()
         addHeaders({QStringLiteral("QSet")});
     }
 
+    if (cfg().qmlRegistration && cfg().singleton) {
+        addHeaders({QStringLiteral("QQmlEngine")});
+    }
+
     if (cfg().singleton) {
         stream() << '\n';
     }
@@ -165,6 +168,14 @@ void KConfigSourceGenerator::createSingletonImplementation()
     stream() << "  return s_global" << cfg().className << "()->q;\n";
     stream() << "}\n\n";
 
+    if (cfg().qmlRegistration && cfg().singleton) {
+        stream() << cfg().className << " *" << cfg().className << "::create(QQmlEngine *, QJSEngine *)\n";
+        stream() << "{\n";
+        stream() << "  QQmlEngine::setObjectOwnership(self(), QQmlEngine::CppOwnership);\n";
+        stream() << "  return self();\n";
+        stream() << "}\n\n";
+    }
+
     if (parseResult.cfgFileNameArg) {
         auto instance = [this](const QString &type, const QString &arg, bool isString) {
             stream() << "void " << cfg().className << "::instance(" << type << " " << arg << ")\n";
@@ -705,15 +716,3 @@ void KConfigSourceGenerator::createSignalFlagsHandler()
 
     stream() << "}\n";
 }
-
-void KConfigSourceGenerator::includeMoc()
-{
-    const QString mocFileName = cfg().baseName + QStringLiteral(".moc");
-
-    if (parseResult.signalList.count() || cfg().generateProperties) {
-        // Add includemoc if they are signals defined.
-        stream() << '\n';
-        stream() << "#include \"" << mocFileName << "\"\n";
-        stream() << '\n';
-    }
-}
